from safepay_python.utils.builder import *
import urllib.parse

class Checkout:
    def __init__(self, config):
        self.config = config

    def create_checkout(self, checkout_details):
        url = build_checkout_url(self.config["environment"])
        params = urllib.parse.urlencode({'beacon': checkout_details['beacon'],
                                         'cancel_url': checkout_details['cancelUrl'],
                                         'env': self.config["environment"],
                                         'order_id': checkout_details['orderId'],
                                         'redirect_url': checkout_details['redirectUrl'],
                                         'source': checkout_details['source'] if 'source' in checkout_details else 'custom',
                                         'webhooks': 'true' if ('webhooks' in checkout_details and checkout_details['webhooks'] == True) else 'false'})
        url += f'?{params}'
        return url




import json
from hashlib import sha256, sha512
import hmac
from safepay_python.utils.secure_compare import *


class Verify:
    def __init__(self, config):
        self.config = config

    def signature(self, body):
        signature = body['sig']
        tracker = body['tracker']
        secret = self.config['v1Secret']
        mac = hmac.new(secret.encode('utf-8'),
                       msg=tracker.encode('utf-8'),
                       digestmod=sha256)
        comptuted_sig = mac.hexdigest()
        comparison_res = secure_compare(signature, comptuted_sig)

        return comparison_res

    def webhook(self, header, body):
        signature = (header['x-sfpy-signature'])
        secret = self.config['webhookSecret']
        payload = json.dumps(
            body['data'], separators=(',', ':'))

        mac = hmac.new(secret.encode('utf-8'),
                       msg=payload.encode('utf-8'),
                       digestmod=sha512)
        comptuted_sig = mac.hexdigest()
        comparison_res = secure_compare(comptuted_sig, signature)
        
        return comparison_res



from safepay_python.utils.constants import *


def build_checkout_url(env):
    if env == Environment.Development.value:
        return CHECKOUT_DEVELOPMENT
    elif env == Environment.Sandbox.value:
        return CHECKOUT_SANDBOX
    else:
        return CHECKOUT_PRODUCTION


def build_api_url(env):
    if env == Environment.Development.value:
        return API_URL_DEVELOPMENT
    elif env == Environment.Sandbox.value:
        return API_URL_SANDBOX
    else:
        return API_URL_PRODUCTION



from safepay_python.utils.builder import *
from safepay_python.utils.validation import *
import requests
import json


class Payments:
    def __init__(self, config):
        self.config = config

    async def create_payment(self, payment_details):
        validate_payment_details(payment_details)
        base_url = build_api_url(self.config['environment'])
        url = '/order/v1/init'
        data = {
            'amount': payment_details['amount'],
            'client': self.config['apiKey'],
            'currency': payment_details['currency'],
            'environment': self.config['environment']
        }

        response = requests.post(url=f"{base_url}{url}", json=data, headers={
                                 'Content-type': 'application/json', 'Accept': 'text/plain'})

        return response.json()



def validate_options(options):

    try:
        options['environment']
    except:
        raise Exception("Environment is missing")

    if validate_environment(options['environment']) == False:
        raise Exception("Environment is invalid")

    try:
        options['apiKey']
    except:
        raise Exception(f"API key is missing for {options['environment']}")

    try:
        options['v1Secret']
    except:
        raise Exception(f"v1 secret is missing for {options['environment']}")

    try:
        options['webhookSecret']
    except:
        raise Exception(
            f"Webhook secret is missing for {options['environment']}")

    return True


def validate_environment(env):

    if env in ["production", "sandbox", "development"]:
        return True
    else:
        return False


def validate_checkout_parameters(params):

    try:
        params['beacon']
    except:
        raise Exception(f"Token is missing")

    try:
        params['cancelUrl']
    except:
        raise Exception(f"Cancel url is missing")

    try:
        params['redirectUrl']
    except:
        raise Exception(f"Redirect url is missing")

    return True


def validate_payment_details(payment_details):

    try:
        payment_details['amount']
    except:
        raise Exception(f"Amount is missing")

    if payment_details['amount'] < 1:
        raise Exception(f"Amount should be greater than 0")

    try:
        payment_details['currency']
    except:
        raise Exception(f"Currency is missing")

    if validate_currency(payment_details['currency']) == False:
        raise Exception(f"Currency not supported")

    return True


def validate_currency(currency):

    if currency in ["PKR", "USD", "AED", "SAR", "CAD", "EUR", "GBP"]:
        return True
    else:
        return False



import hmac
import six


def secure_compare(a, b):
    """
    Return 'a == b'.  This function uses an approach designed to prevent
    timing analysis, making it appropriate for cryptography.
    a and b must both be of the same type: either str (ASCII only),
    or any bytes-like object.
    """

    def utf8(value):
        if six.PY2 and isinstance(value, six.text_type):
            return value.encode('utf-8')
        else:
            return value

    return hmac.compare_digest(utf8(a), utf8(b))



from enum import Enum

API_URL_PRODUCTION = 'https://api.getsafepay.com'
API_URL_SANDBOX = 'https://sandbox.api.getsafepay.com'
API_URL_DEVELOPMENT = 'https://dev.api.getsafepay.com'

CHECKOUT_PRODUCTION = 'https://getsafepay.com/checkout/pay'
CHECKOUT_SANDBOX = 'https://sandbox.api.getsafepay.com/checkout/pay'
CHECKOUT_DEVELOPMENT = 'https://dev.api.getsafepay.com/checkout/pay'


class Environment(Enum):
    Production = 'production'
    Sandbox = 'sandbox'
    Development = 'development'



from safepay_python.utils.builder import *
from safepay_python.utils.validation import *
from safepay_python.resources.checkout import *
from safepay_python.resources.payments import *
from safepay_python.resources.verify import *
import asyncio


class Safepay:
    def __init__(self, config):
        validate_options(config)
        self.config = config

    def get_checkout_url(self, checkout_details):
        checkout = Checkout(self.config)
        validate_checkout_parameters(checkout_details)
        return checkout.create_checkout(checkout_details)

    def set_payment_details(self, payment_details):

        async def paymentFucntion(payment_details):
            payment = Payments(self.config)
            value = await payment.create_payment(payment_details)
            return value

        payment_value = asyncio.run(paymentFucntion(payment_details))
        return payment_value

    def is_signature_valid(self, signature_body):
        verify = Verify(self.config)
        return verify.signature(signature_body)

    def is_webhook_valid(self, webhook_header, webhook_body):
        verify = Verify(self.config)
        return verify.webhook(webhook_header, webhook_body)